/*
 *    TODO.java
 *    Copyright (C) 2000 Stuart Yeates 
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

// the package we're in
package org.nzdl.gsdl.classification;

// java standard library classes used
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.Serializable;
import java.lang.Cloneable;
import java.util.Vector;
import java.util.Hashtable;
import java.util.Enumeration;
import java.util.Properties;

/**
 * The Library of Congress Classification System
 * 
 * @author stuart yeates (say1@cs.waikato.ac.nz)
 * @version $Revision: 2194 $
 * @see org.nzdl.gsdl.classification.Classification
 * @see org.nzdl.gsdl.classification.ClassificationSystem
 * @see org.nzdl.gsdl.classification.LibraryOfCongressClassification
 * @see java.io.Serializable
 * @see java.lang.Cloneable
 */

/**
 * A  Dewey Decimal Subject Classification Category
 *
 * @author stuart yeates (say1@cs.waikato.ac.nz)
 * @version $Revision: 2194 $
 * @see org.nzdl.gsdl.classification.ClassificationSystem
 * @see java.io.Serializable
 * @see java.lang.Cloneable
 * @see
 */
public class LibraryOfCongressClassification implements Classification {
  LibraryOfCongressClassificationSystem system = null;
  LibraryOfCongressClassification parent = null;
  String key = null;
  String value = null;
  String parentValue = null;  // parentValue  + " --- " + myValue == value
  String myValue = null;      //
  Vector subClassifications = null;
  int level;

  LibraryOfCongressClassification(LibraryOfCongressClassificationSystem system,
			     LibraryOfCongressClassification parent,
			     int level,
			     String key, 
			     String value)
  {
    this.system = system;
    this.parent = parent;
    this.key = key;
    this.value = value;
    this.level = level;
    subClassifications = new Vector();
    for(int i = 0; i < 10;i++)
      subClassifications.add(null);
  }

  boolean add (String key, String value) {
    if (key.length() < level+1)
      throw new Error("blarg " + key.length() + " " + level);
    int next = Integer.parseInt(key.substring(level,level+1));
    if (subClassifications.elementAt(next) == null) {
      LibraryOfCongressClassification classification = 
	new LibraryOfCongressClassification(system,
				       this,
				       level+1,
				       key,
				       value);
      subClassifications.setElementAt(classification,next);
      return true;
    } else {
      LibraryOfCongressClassification classification = 
	(LibraryOfCongressClassification) subClassifications.elementAt(next);
     return classification.add(key,value);
    }
  }

  void clean() {
    String common = null;

    if (value == null) {
      for (int i = 0; i < 10; i++){
	LibraryOfCongressClassification classification = 
	  (LibraryOfCongressClassification) subClassifications.elementAt(i);
	if (classification != null){
	  classification.clean();
	  if (common == null)
	    common = clean (classification.getFullTextualDescription());
	  else {
	    String diff = clean (classification.getFullTextualDescription());
	    int min = Math.min(common.length(),diff.length());
	    StringBuffer com = new StringBuffer(min);
	    int j = 0;
	    for (j = 0; j < min && (diff.charAt(j) == common.charAt(j)); j++){
	      com.append(diff.charAt(j));
	    }
	    common = com.toString(); 
	  }
	}
      }
      value = common;
    }
    if (parent == null)
      parentValue = "";
    else
      parentValue = parent.value;
    myValue = clean (value.substring(parentValue.length(),value.length())); 
  }

  String clean(String in) {
    int first = 0;
    int last = in.length();
    for (; first < last-1 && 
	   (in.charAt(first) == ' ' || 
	    in.charAt(first) == '-'); first++)
      ;
    for (; last > 1 && 
	   (in.charAt(last-1) == ' ' || 
	    in.charAt(last-1) == '-'); last--)
      ;

    return in.substring(first,last);
  }


  /** What is the textual description of this Classification? */
  public Classification getClassificationFor(String string) {
    if (key.length() < level)
      return this;
    
    int next = Integer.parseInt(key.substring(level,level+1));
    if (subClassifications.elementAt(next) == null) {
      return this;
    } else {
      LibraryOfCongressClassification classification = 
	(LibraryOfCongressClassification) subClassifications.elementAt(next);
      return classification.getClassificationFor(string);
    }
  }
  
  /** What is the textual description of this Classification? */
  public String getTextualDescription() {
    return value;
  }
    /** Verbose textual description */
  public String getFullTextualDescription() {
    if (parent == null)
      return value;
    else
      return parent.getFullTextualDescription() + value;
  }
  /** What is the code representing this Classification */
  public String getLabel() {
    return key;
  }
  /** What System is this Classification part of? */
  public ClassificationSystem getSystem(){
    return system;
  }

  /** Does this Classification contain this sub-Classification? */
  public boolean contains(Classification instance){
    return false;
  }
  /** What direct sub-Classifications does this Classification have? */
  public Enumeration getSubClassifications() {
    return subClassifications.elements();
  }
  /** The parent Classification */
  public Classification getParent() {
    return parent;
  }
  
}
