/*
 *    LibraryOfCongressClassificationSystem.java
 *    Copyright (C) 2000 Stuart Yeates 
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

// the package we're in
package org.nzdl.gsdl.classification;

// java standard library classes used
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.Serializable;
import java.lang.Cloneable;
import java.util.Vector;
import java.util.Hashtable;
import java.util.Enumeration;
import java.util.Properties;

/**
 * The Library of Congress Classification System
 * 
 * @author stuart yeates (say1@cs.waikato.ac.nz)
 * @version $Revision: 2194 $
 * @see org.nzdl.gsdl.classification.Classification
 * @see org.nzdl.gsdl.classification.ClassificationSystem
 * @see org.nzdl.gsdl.classification.LibraryOfCongressClassification
 * @see java.io.Serializable
 * @see java.lang.Cloneable
 */

public class LibraryOfCongressClassificationSystem 
  implements ClassificationSystem {
  
  static LibraryOfCongressClassificationSystem instance = null;
  
  static final protected String name = "Dewey Decimal Classification System";
  LibraryOfCongressClassification root = null;
  
  protected LibraryOfCongressClassificationSystem() {
    root = new LibraryOfCongressClassification(this, null,0,"",getName());
    init();
    root.clean();
  }
  static public LibraryOfCongressClassificationSystem getInstance() {
    if (instance == null)
      instance = new LibraryOfCongressClassificationSystem();
    return instance;
  }

  protected boolean add (String key, String value) {
    return root.add(key,value);
  }
  
  protected void init() {
  }
  /** */
  public Classification getClassificationFor(String string) {
    return root.getClassificationFor(string);
  }


  /** Returns the name of the ClassificationSystem*/
  public String getName() { return name; };

  /** 
   * Returns an enumeration of all top-level Classifications 
   * (or all Classifications if this is a non-heirarchical 
   * ClassificationSystem) 
   */
  public Enumeration getClassifications() {return root.getSubClassifications();};
 
  /**
   * Parse a string containing one or more Classifications and
   * return an Enumeration of them.
   */
  public Enumeration  parse(String rawDesc) {
    Vector vec = new Vector();
    StringBuffer tmp = new StringBuffer();
    for(int i = 0; i <rawDesc.length();i++) {
      if (Character.isDigit((char)i) || i == '.') {
	tmp.append(i);
      } else {
	if (tmp.length() != 0)
	  vec.add(getClassificationFor(tmp.toString()));
	tmp.delete(0,tmp.length());
      }
    }      
    return vec.elements();

  };

  public static void main(String args[]) throws IOException
  {
    LibraryOfCongressClassificationSystem system = 
      new LibraryOfCongressClassificationSystem();
  }

}
